<?php

namespace Drupal\flexible_access;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityInterface;
use Drupal\node\Entity\Node;
use Drupal\user\RoleInterface;
use Drupal\flexible_access\Helper;

/**
 * Class FlexibleAccessGroupRelationshipAccessUpdate.
 *
 * @package Drupal\flexible_access
 */
class FlexibleAccessGroupRelationshipAccessUpdate {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The helper service.
   *
   * @var \Drupal\flexible_access\Helper
   */
  protected $helperService;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler, Helper $helper_service) {
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler = $module_handler;
    $this->helperService = $helper_service;
  }

  /**
   * Update Group content after Group changed.
   *
   * @param \Drupal\Core\Entity\EntityInterface $group
   *   The Group we've updated.
   * @param array $changed_visibility
   *   The Group's old visibility.
   * @param array $new_options
   *   The Group's new visibility options.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public static function batchUpdateGroupRelationshipAccess(EntityInterface $group, string $new_access) {
    // Set it up as a batch. We need to update visibility.
    // Load all the GroupContentEntities.
    // Memberships don't need an update.
    $entities = $group->getRelatedEntities();


    $num_operations = count($entities);
    $operations = [];

    // Update ContentVisibility.
    // As per documentation each entity has it's own operation.
    for ($i = 0; $i < $num_operations; $i++) {
      $operations[] = [
        '\Drupal\flexible_access\FlexibleAccessGroupRelationshipAccessUpdate::updateAccess',
        [
          $entities[$i],
          $new_access,
        ],
      ];
    }

    // Provide all the operations and the finish callback to our batch.
    $batch = [
      'title' => t('Updating Group Content...'),
      'operations' => $operations,
      'finished' => '\Drupal\flexible_access\FlexibleAccessGroupRelationshipAccessUpdate::updateAccessFinishedCallback',
    ];

    batch_set($batch);
  }

  /**
   * Update visibility for all Group Content based on a new group type.
   *
   * @param \Drupal\node\Entity\Node|\Drupal\group\Entity\GroupMembership|\Drupal\group\Entity\GroupRelationshipInterface $entity
   *   The content we are updating.
   * @param array $new_options
   *   The Group's new visibility options.
   * @param array $context
   *   Passed on by reference.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public static function updateAccess($entity, string $new_access, array &$context) {

    $flexible_access_field = \Drupal::service('flexible_access.helper')->getFlexibleAccessField($entity);

    if (isset($flexible_access_field)) {
      if ($entity instanceof Node && $entity->hasField($flexible_access_field)) {
        $entity->set($flexible_access_field, $new_access);
        $entity->save();
      }
    }

   // Make sure our GroupRelationship referenced entities also get invalidated.
    $tags = $entity->getCacheTagsToInvalidate();
    Cache::invalidateTags($tags);

    // Add referenced entity to results. Might want to add it to the result.
    $context['results'][] = $entity;

    // Optional message displayed under the progressbar.
    $context['message'] = t('Updating group content (@id)', ['@id' => $entity->id()]);
  }

  /**
   * Callback for finished batch events.
   *
   * @param bool $success
   *   TRUE if the update was fully succeeded.
   * @param array $results
   *   Contains individual results per operation.
   * @param array $operations
   *   Contains the unprocessed operations that failed or weren't touched yet.
   */
  public static function updateAccessFinishedCallback($success, array $results, array $operations) {
    $messenger = \Drupal::messenger();
    if ($success) {
      // Here we could do something meaningful with the results.
      // We just display the number of nodes we processed...
      $messenger->addStatus(t('Flexible access of content item(s) updated.'));
    }
    else {
      // An error occurred.
      // $operations contains the operations that remained unprocessed.
      $error_operation = reset($operations);
      $messenger->addError(
        t('An error occurred while processing @operation with arguments : @args',
          [
            '@operation' => $error_operation[0],
            '@args' => print_r($error_operation[0], TRUE),
          ]
        )
      );
    }
  }
}


