<?php

declare(strict_types=1);

namespace Drupal\flexible_access\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\flexible_access\Helper;
use Drupal\flexible_access\Plugin\FlexibleAccessItemManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Flexible Access form.
 */
final class FlexibleAccessItemAddForm extends FormBase {

  /**
   * The helper service.
   * 
   * @var \Drupal\flexible_access\Helper
   */
  protected $helperService;

  /**
   * The flexible access item manager.
   * 
   * @var \Drupal\flexible_access\Plugin\FlexibleAccessItemManager
   */
  protected $flexibleAccessItemManager;


  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create(
      $container
    );    
    $instance->setHelperService($container->get('flexible_access.helper'));
    $instance->setFlexibleAccessItemManager($container->get('plugin.manager.flexible_access_item.processor'));
    return $instance;
  }  

  /**
   * Sets current user.
   */
  public function setHelperService(Helper $helper_service): void {
    $this->helperService = $helper_service;
  }    

  /**
   * Sets flexible access item manager.
   */
  public function setFlexibleAccessItemManager(FlexibleAccessItemManager $flexible_access_item_manager): void {
    $this->flexibleAccessItemManager = $flexible_access_item_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'flexible_access_flexible_access_item_add';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $header = [
      'col1' => $this->t('Label'),
      'col2' => $this->t('Information'),
    ];    

    $rows = [];

    // Scan for plugins.
    $plugins = $this->flexibleAccessItemManager->getDefinitions();
    if (!empty($plugins)) {
      foreach ($plugins as $plugin_id => $plugin) {
        // Check if the plugin is already in the database.
        if ($this->helperService->getFlexibleAccessItem($plugin_id)) {
          $info = $this->t('installed');
        }
        else {
          $info = $this->t('available');
        }

        $rows[$plugin_id] = [
          'col1' => $plugin['label'],
          'col2' => $info,
        ];
      }
    }
    
    $form['existing_plugins'] = [
      '#type' => 'tableselect',
      '#header' => $header,
      '#options' => $rows,
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Add selected plugins'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // @todo Validate the form here.
    // Example:
    // @code
    //   if (mb_strlen($form_state->getValue('message')) < 10) {
    //     $form_state->setErrorByName(
    //       'message',
    //       $this->t('Message should be at least 10 characters.'),
    //     );
    //   }
    // @endcode
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $processed = 0;
    $form_state->setRedirect('entity.flexible_access_item.collection');    
    $plugins = $form_state->getValue('existing_plugins');
    foreach ($plugins as $plugin_id => $value) {
      if (!$this->helperService->getFlexibleAccessItem($plugin_id)) {
        $processed++;
        $this->helperService->createFlexibleAccessItem($plugin_id); 
      }
    }
    if ($processed > 0) {
      $this->messenger()->addStatus($this->t('Great! New plugins are instaled.'));
    }
    else {
      $this->messenger()->addStatus($this->t('There were no new plugins to install.'));
    }
  }

}
