<?php

declare(strict_types=1);

namespace Drupal\flexible_access\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\flexible_access\Entity\FlexibleAccessItem;
use Drupal\flexible_access\Helper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Flexible access item form.
 */
final class FlexibleAccessItemForm extends EntityForm {


  /**
   * The helper service.
   * 
   * @var \Drupal\flexible_access\Helper
   */
  protected $helperService;
  

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();
    $instance->setHelperService($container->get('flexible_access.helper'));
    return $instance;
  }  

  /**
   * Set the helper service.
   *
   * @param \Drupal\flexible_access\Helper $helper
   *   The helper service.
   */
  public function setHelperService($helper) {
    $this->helperService = $helper;
  }     

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {

    $form = parent::form($form, $form_state);

    $field_options = $this->helperService->getSupportedUserReferenceFields();

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $this->entity->status(),
    ];    

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 32,
      '#default_value' => $this->entity->label(),
      '#required' => TRUE,
      '#description' => $this->t('Label for the flexible access item. As it will also been used for the machine name, we recommend shorter labeling.'),
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => [FlexibleAccessItem::class, 'load'],
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $this->entity->get('description'),
      '#description' => $this->t('Description for the flexible access item.'),
    ];

    /*

    $form['roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#options' => $this->helperService->getSupportedRoles(),
      //'#default_value' => $this->entity->getRoles() ?? [],
    ];  

    */

    $form['field'] = [
      '#type' => 'select',
      '#title' => $this->t('Field'),
      '#options' => $field_options,
      '#default_value' => $this->entity->get('field') ?? '',
      '#description' => $this->t('If selected, the users of that field will have access to the entity.'),
      '#access' => FALSE,
    ];    

    $form['operations'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Operations'),
      '#options' => $this->helperService->getSupportedOperations(),
      '#default_value' => $this->entity->get('operations') ?? ['view'],
      '#access' => FALSE,
    ];

    $form['dependency'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Dependency'),
      '#default_value' => $this->entity->get('dependency') ?? '',
      '#description' => $this->t('Dependency for the flexible access item. This would be a machine name of a module.'),
    ];

    $form['weight'] = [
      '#type' => 'number',
      '#title' => $this->t('Weight'),
      '#default_value' => $this->entity->get('weight') ?? 0,
      '#description' => $this->t('Weight for the flexible access item.'),
    ];
   

    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {

    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match($result) {
        \SAVED_NEW => $this->t('Created new example %label.', $message_args),
        \SAVED_UPDATED => $this->t('Updated example %label.', $message_args),
      }
    );
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

}
