<?php

namespace Drupal\flexible_access\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\flexible_access\Plugin\FlexibleAccessItemManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'flexible_access_default' widget.
 *
 * @FieldWidget(
 *     id="flexible_access_default",
 *     label=@Translation("Default"),
 *     field_types={
 *         "flexible_access_field"
 *     }
 * )
 */
class FlexibleAccessDefaultWidget extends WidgetBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The flexible access item manager.
   * 
   * @var \Drupal\flexible_access\Plugin\FlexibleAccessItemManager
   */
  protected $flexibleAccessItemManager;  
  
  
  /**
   * {@inheritdoc}
   */  
   public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create(
      $container,
      $configuration,
      $plugin_id,
      $plugin_definition
    );    
    $instance->setEntityTypeManager($container->get('entity_type.manager'));
    $instance->setFlexibleAccessItemManager($container->get('plugin.manager.flexible_access_item.processor'));
    return $instance;
  }  

  /**
   * Sets entity type manager.
   */
  public function setEntityTypeManager(EntityTypeManagerInterface $entity_type_manager): void {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Sets flexible access item manager.
   */
  public function setFlexibleAccessItemManager(FlexibleAccessItemManager $flexible_access_item_manager): void {
    $this->flexibleAccessItemManager = $flexible_access_item_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $options = $this->getFilteredEntities();

    $element['#attached']['library'][] = 'flexible_access/flexible_access_widget_styles';

    $default_value = !empty($items[0]->target_id) 
    ? $items[0]->target_id
    : NULL;

    $element['target_id'] = [
      '#type' => $this->fieldDefinition->getFieldStorageDefinition()->getCardinality() == 1 ? 'radios' : 'checkboxes',
      '#options' => $options,
      '#default_value' => $default_value,
      '#description' => $this->t('Select the access item(s) that should be applied to this entity.'),
      '#theme' => 'radios_icon_labels',
      '#attributes' => ['class' => ['flexible-access-radios']],
    ];

    return $element;
  }

  /**
   * Retrieves entities of type 'flexible_access_item' but filters out disabled ones.
   *
   * @return array
   *   An associative array of entity labels indexed by entity IDs.
   */
  protected function getFilteredEntities() {
    $storage = $this->entityTypeManager->getStorage('flexible_access_item');
    $query = $storage->getQuery()
      ->condition('status', TRUE) // Only get enabled entities
      ->sort('weight', 'ASC');

    $ids = $query->execute();
    $entities = $storage->loadMultiple($ids);

    $options = [];
    foreach ($entities as $entity) {
      $icon = $this->getIconFromPlugin($entity->id());
      $options[$entity->id()] = $icon . $entity->label() . '<span class="flexible-access-description form-item__description">' . $entity->getDescription() . '</span>';
    }

    return $options;
  }  

  protected function getIconFromPlugin($plugin_id) {

    $icon = '';

    $plugin_manager = $this->flexibleAccessItemManager;
    if ($plugin_manager->hasDefinition($plugin_id)) { 
      $plugin = $plugin_manager->createInstance($plugin_id);
      $icon = $plugin->getIcon();
    }

    return $icon;

  }



}